'''
Created on 2013/7/3

@author: kent
'''
import os
import sys
import time
import pprint
import logging
import csv

from junit_xml import TestSuite, TestCase

logger = logging.getLogger('Reporter')


class Reporter:
    def __init__(self, xml_filename=None):
        self.summary = {}
        self.ran_cases = 0
        self.passed_cases = 0
        self.failed_cases = 0
        self.html_template = ''
        self.report_create_time = 'XX'
        self.reports_dir = os.getcwd() + '/Reports/'
        self.latest_reports_dir = os.path.join(self.reports_dir, 'Latest')
        self.xml_filename = None if xml_filename is None else xml_filename

    def _create_html_head(self):
        html_head = ' \
            <meta charset="utf-8" /> \
            <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1" /> \
            <meta name="description" content="" /> \
            <meta name="author" content="kent chen" /> \
            <meta name="viewport" content="width=device-width; initial-scale=1.0" /> \
            <link href="http://netdna.bootstrapcdn.com/twitter-bootstrap/2.3.2/css/bootstrap-combined.min.css" rel="stylesheet"> \
            <script src="http://netdna.bootstrapcdn.com/twitter-bootstrap/2.3.2/js/bootstrap.min.js"></script>'
        return html_head

    def _create_summary_table(self, case_classify, test_summary):
        if not test_summary:
            html_table =''
        else:
            html_table = ' \
                <table class="table table-hover table-bordered" style="width:200px"> \
                <tbody> \
                <tr class="info"><td>Ran Cases</td><td><strong>%s</strong></td></tr> \
                <tr class="success"><td>Passed</td><td><strong>%s</strong></td></tr> \
                <tr class="error"><td>Failed</td><td><strong>%s</strong></td></tr> \
                </tbody> \
                </table>' % (test_summary['ran_cases'], test_summary['passed_cases'], test_summary['failed_cases'])
        return html_table

    def _create_html_table(self, test_result):
        table_body_content = ''
        keys = test_result.keys()
        keys.sort()
        for case_id in keys:
            if test_result[case_id][0] == 'Fail':
                table_body_content =  table_body_content + \
                                      '<tr><td>%s</td><td style="background-color:#F2DEDE">%s</td><td>%s</td><td>%s</td></tr>' % \
                                      (case_id, str(test_result[case_id][0]), str(test_result[case_id][1]), str(test_result[case_id][2]))
            else:
                table_body_content =  table_body_content + \
                                      '<tr><td>%s</td><td style="background-color:#D0E9C6">%s</td><td>%s</td><td>%s</td></tr>' % \
                                      (case_id, str(test_result[case_id][0]), str(test_result[case_id][1]), str(test_result[case_id][2]))

        html_table = ' \
            <table class="table table-hover"> \
            <thead><tr><th>Case ID</th><th>Test Result</th><th>Log Message</th><th>Duration Time</th></tr></thead> %s \
            </table>' % table_body_content

        return html_table

    def _create_report_file(self, filename, content):
        report_file_name = os.path.join(self.latest_reports_dir, filename + '.html')

        if not os.path.exists(self.reports_dir):
            os.mkdir(self.reports_dir)

        if not os.path.exists(self.latest_reports_dir):
            os.mkdir(self.latest_reports_dir)

        with open(report_file_name, 'wb') as fh:
            fh.write(content)

    def _generate_html_file(self, case_classify, test_result, test_summary):
        head = self._create_html_head()
        table = self._create_html_table(test_result)
        summary_table = self._create_summary_table(case_classify, test_summary)
        style = 'div#main {width: 90%; margin: auto}'

        html = '''
<html>
    <head>
        <style>%s</style>
        <title>%s</title>
        %s
    </htad>
    <body>
        <div id="main">
            <h1>%s</h1>
            <hr>
            <h3>Summary</h3>
            %s
            <hr>
            <h3>Detail</h3>
            %s
        </div>
    </body>
</html>
        ''' % (style, case_classify, head, case_classify, summary_table, table)

        self._create_report_file(case_classify, html)

    def _generate_summary_html_file(self, test_summary):
        summary_table = ''
        total_pass = 0
        total_fail = 0
        total_ran = 0
        for case_classify in test_summary:
            total_pass = total_pass + test_summary[case_classify]['passed_cases']
            total_fail = total_fail + test_summary[case_classify]['failed_cases']
            total_ran = total_ran + test_summary[case_classify]['ran_cases']
            summary_table = summary_table + \
                            '<tr><td>%s</td><td style="background-color:#D0E9C6">%s</td><td style="background-color:#F2DEDE">%s</td><td>%s</td></tr>' % \
                            (case_classify,
                                str(test_summary[case_classify]['passed_cases']),
                                str(test_summary[case_classify]['failed_cases']),
                                str(test_summary[case_classify]['ran_cases']))

        html_table = ' \
            <table class="table table-hover"> \
            <thead><tr><th>Classify<th>Pass</th><th>Fail</th><th>Ran Cases</th></tr></thead> %s \
            <tfoot><tr><td>Total</td><td>%s</td><td>%s</td><td>%s</td></tr></tfoot> \
            </table>' % (summary_table, str(total_pass), str(total_fail), str(total_ran))

        head = self._create_html_head()
        style = 'div#main {width: 90%; margin: auto} tfoot { font-weight:bold }'

        html_code = '''
<html>
    <head>
        <style>%s</style>
        <title>Total Summary Result</title>
        %s
    </htad>
    <body>
        <div id="main">
            <h1>Summary</h1>
            %s
        </div>
    </body>
</html>
        ''' % (style, head, html_table)

        self._create_report_file('SummaryReport', html_code)

    def _output_result_to_csv(self, test_result):
        import shutil
        csv_content = []
        for case_classify in test_result.keys():
            csv_file_path = test_result[case_classify]['csv_file_path']
            csv_file_path_basename = os.path.basename(csv_file_path)
            report_csv_file_path = os.path.join(self.latest_reports_dir, csv_file_path_basename)
            shutil.copy(csv_file_path, report_csv_file_path)

            with open(report_csv_file_path, 'rb') as fh:
                csv_content = [row for row in csv.reader(fh, delimiter=',') if len(row) > 0]

            for index, row in enumerate(csv_content):
                if index != 0:
                    case_id = csv_content[index][1]
                    if case_id in test_result[case_classify]['result'].keys():
                        csv_content[index][5] = test_result[case_classify]['result'][case_id][0]
                        csv_content[index][6] = test_result[case_classify]['result'][case_id][1]
                        csv_content[index][7] = test_result[case_classify]['result'][case_id][2]

            with open(report_csv_file_path, 'wb') as fh:
                csv_writer = csv.writer(fh, delimiter=',')
                try:
                    for row in csv_content:
                        csv_writer.writerow(row)
                    os.remove(csv_file_path_backup)  # remove the backup if update csv success.
                except:
                    logger.error('Output to csv file fail!')

    def _get_summary_dict(self, test_result):
        summary_dict = {}
        for classify in test_result:
            summary_dict[classify] = {
                "passed_cases": test_result[classify]['summary']['passed_cases'],
                "failed_cases": test_result[classify]['summary']['failed_cases'],
                "ran_cases": test_result[classify]['summary']['ran_cases']
            }

        return summary_dict

    def _output_normal(self, test_result):
        # Need refactor
        if test_result == {}:
            print '[what?!] there are not any test result, what is the test case id?'
        else:
            xml_test_suites = []
            summary_dict = self._get_summary_dict(test_result)
            self.report_create_time = str(time.strftime('%Y%m%d_%H%M%S', time.localtime()))
            for case_classify in test_result.keys():
                xml_test_cases = []
                if 'result' in test_result[case_classify].keys():
                    # Generate HTML report
                    self._generate_html_file(
                        case_classify, test_result[case_classify]['result'],
                        test_result[case_classify]['summary'])

                    # Save the result into the CSV
                    self._output_result_to_csv(test_result)

                    # Show in Console
                    print '{0} {1} {2}'.format('='*16, case_classify, '='*16)
                    test_case_result = test_result[case_classify]['result']
                    for case_id in test_case_result.keys():
                        print '[{0}][{1}] {2}, {3}, {4}'.format(case_classify, case_id,
                                                                test_case_result[case_id][0],
                                                                test_case_result[case_id][1],
                                                                str(test_case_result[case_id][2]))

                        # Produce xml file
                        test_case = TestCase(case_id, case_classify, int(test_case_result[case_id][2]))
                        if test_case_result[case_id][0] == 'Fail' or test_case_result[case_id][0] == 'Error':
                            try:
                                test_case.add_failure_info('msg' + test_case_result[case_id][1])
                            except:
                                test_case.add_failure_info('msg' + str(test_case_result[case_id]))

                        xml_test_cases.append(test_case)

                    xml_test_suites.append(TestSuite(case_classify, xml_test_cases))
                    with open(os.path.join(self.latest_reports_dir, case_classify + '.xml'), 'w') as f:
                        TestSuite.to_file(f, xml_test_suites, prettyprint=True)

            self._generate_summary_html_file(summary_dict)
            print '{0} {1} {2}'.format('='*16, 'Summary', '='*16)
            pprint.pprint(summary_dict)

    def output_report(self, test_result):
        self._output_normal(test_result)



#!/usr/bin/python
import os
import sys
import logging
from optparse import OptionParser
import re

import Engine.TestEngine as TestEngine
import Engine.parser as Parser
import Engine as engine
from Engine.config import VariablesPool


LOGGING_LEVELS = {'critical': logging.CRITICAL,
                  'error': logging.ERROR,
                  'warning': logging.WARNING,
                  'info': logging.INFO,
                  'debug': logging.DEBUG}


def parse_variable(variables):
    if re.search('^\w+\:\w+,{0,1}', variables):
        variable_dict = {}
        variable_list = variables.split(',')

        for vairable in variable_list:
            vairable_pair = vairable.split(':')
            variable_dict.update({vairable_pair[0]: vairable_pair[1]})
            setattr(VariablesPool, vairable_pair[0], vairable_pair[1])
    else:
        raise Exception('The format of the string to variable is wrong.')


def main():
    sys.path.append(os.getcwd())
    parser = OptionParser(usage="usage: %prog [options][arg]")
    parser.add_option('-d', '--debug',
                      action='store',
                      type='string',
                      dest='debug_flag',
                      help='Turn on the debug mode [debug|info|warning|error]. Ex: $python threat_tester.py -d debug -c Dummy')
    parser.add_option('-c', '--caseid',
                      action='store',
                      type='string',
                      dest="caseid_prefix",
                      help="Run the specific test case by ID or prefix of test case ID.")
    parser.add_option('-s', '--csv',
                      action='store',
                      type='string',
                      dest="run_csv_path",
                      help="Run all test case in a specific csv file.")
    parser.add_option('-l', '--csvlist',
                      action='store',
                      type='string',
                      dest="csv_list",
                      help="Run all csv files via list in a file. It can exectue the csv by order from top to bottom.")
    parser.add_option('-a', '--all',
                      action='store_true',
                      dest="run_all_flag",
                      help="Run all the test cases")
    parser.add_option("-g", "--gen",
                      action='store',
                      type='string',
                      dest="csv_file_path",
                      help="Generate the template of test scripts. Ex: $python threat_tester.py TestSuites/Dummy.csv")
    parser.add_option("-t", "--test",
                      action='store_true',
                      dest="test_flag",
                      default=False,
                      help="For develope use")
    parser.add_option("-x", "--xml",
                      action='store',
                      dest="xml_filename",
                      help="Output the xml file with junit xml format.")
    parser.add_option("-v", "--variables",
                      action='store',
                      dest="variables",
                      help="Variables with 'var1:AAA,var2:BBB'")
    (options, args) = parser.parse_args()

    if options.variables:
        parse_variable(options.variables)

    if options.debug_flag:
        # -d
        logLevel = LOGGING_LEVELS.get(options.debug_flag)
        logging.basicConfig(format='[%(levelname)-6s][%(name)s]:%(message)s', level=logLevel)
        logging.info("Turn on the debug mode!")
    else:
        logging.basicConfig(format='[%(levelname)-6s][%(name)s]:%(message)s', level=logging.WARN)

    if options.caseid_prefix:
        # -c
        runner = TestEngine.Runner(xml_filename=options.xml_filename)
        runner.run(options.caseid_prefix)
    elif options.run_csv_path:
        # -s
        runner = TestEngine.Runner(test_suite_csv=options.run_csv_path, xml_filename=options.xml_filename)
        runner.run_all()
    elif options.csv_list:
        # -l
        with open(options.csv_list, 'rb') as fh:
            for line in fh:
                run_csv_path = line.rstrip('\r\n')
                runner = TestEngine.Runner(test_suite_csv=run_csv_path, xml_filename=options.xml_filename)
                runner.run_all()

    elif options.csv_file_path:
        # -g
        arg = options.csv_file_path
        parser = Parser.TestCaseParser()
        testCaseSuites = parser.parse_from_csv([arg])
        TestEngine.GenerateTestCase(testCaseSuites)
    elif options.test_flag:
        # -t
        testCaseSuites, caseList, csvFileList = engine.ParseFromCSV()
        # Tester.GenerateTestCase(testCaseSuites, caseList, csvFileList)
    elif options.run_all_flag:
        # -a
        runner = TestEngine.Runner(['all'])
        runner.run_all()
    else:
        parser.print_help()


if __name__ == "__main__":
    main()



import logging
import traceback
import json
import urllib2
import urllib
import socket

logger = logging.getLogger('Feedback')


class Feedback:
    def __init__(self, task_id):
        from config import FEEDBACK_SERVER

        self.task_id = task_id
        self.server_url = FEEDBACK_SERVER['server_url']
        self.reports_url = self.server_url + FEEDBACK_SERVER['reports_path']
        self.status_url = self.server_url + FEEDBACK_SERVER['status_path']
        self.hostname = socket.gethostname()
        try:
            urllib2.urlopen(self.server_url, timeout=1)
            self.server_down = False
        except:
            logger.warn('It can not connect to feedback server!')
            self.server_down = True

    def _notify_running(self):
        """statistic: [passed, failed, error, total_ran, run_time, run_status]
        """
        request = {"statistic": json.dumps([0, 0, 0, 0, 0, 1]), "hostname": self.hostname, "task_name": self.task_id}
        encode_data = urllib.urlencode(request)

        # data = 'dashboard={"hostname":"%s","statistic":[0,0,0,1]}' % self.hostname
        try:
            response = urllib2.urlopen(self.status_url, encode_data)
        except:
            response = None
            logger.error('Network configuration may be wrong!')
            logger.error('Server url: %s' % self.status_url)
            logger.error('Send data: %s' % str(encode_data))
            logger.error(traceback.format_exc())

        logger.debug('[notify_running]: %s' % response)

    def _notify_end(self, test_summary):
        pass

    def _feedback_reports(self, test_result):
        #data = 'data=\"%s\",hostname=\"%s\"' % (str(test_result), self.hostname)

        request = { "data": json.dumps(test_result), "hostname": self.hostname, "task_id": self.task_id}
        encode_data = urllib.urlencode(request)

        logger.debug('url: %s' % self.reports_url)
        logger.debug('data: %s' % request)

        try:
            response = urllib2.urlopen(self.reports_url, encode_data)
        except:
            response = None
            logger.error(traceback.format_exc())
            logger.error('Network configuration may be wrong!')

        logger.debug('[feedback_report]: %s' % response)

    def feedback_to_server(self, test_result):
        if self.server_down is False:
            if test_result == {}:
                # start running
                self._notify_running()
            else:
                # finished the test
                self._feedback_reports(test_result)


'''
Created on 2012/6/24

@author: kent
'''
import os
import re
import logging
import traceback
import time
from importlib import import_module

from parser import TestCaseParser
from reporter import Reporter
from feedback import Feedback

logger = logging.getLogger('TestEngine')


class Runner:
    def __init__(self, test_suite_csv=None, task_id=None, xml_filename=None):
        parser = TestCaseParser()
        self.test_suites = parser.parse_from_csv(test_suite_csv)
        self.reporter = Reporter()
        self.feedback = Feedback(task_id)
        self.test_result = {}
        self.test_summary = {}

    def _store_csv_path(self, case_classify, csv_file_path):
        """Store the csv path into test_result dict.
        """
        if case_classify in self.test_result.keys():
            self.test_result[case_classify]['csv_file_path'] = csv_file_path

    def _store_result(self, case_classify, case_id, run_result, log_message, passed_num, failed_num, run_time):
        """Store result related information into test_result dict.
        """
        if run_result is True:
            run_result = 'Pass'
            passed_num += 1
        elif run_result == 'Error':
            run_result = 'Error'
            failed_num += 1
        elif run_result is False:
            failed_num += 1
        else:
            # this include "not run" case
            pass

        if case_classify not in self.test_result.keys():
            self.test_result[case_classify] = {}

        if 'result' not in self.test_result[case_classify].keys():
            self.test_result[case_classify]['result'] = {}

        self.test_result[case_classify]['result'].update({case_id: [str(run_result), log_message, str(run_time)]})
        logger.debug('Test Result: {0}'.format(str(run_result)))

        return passed_num, failed_num

    def _store_summary(self, case_classify, ran_cases, passed_cases, failed_cases, total_run_time):
        if case_classify in self.test_result.keys():
            self.test_result[case_classify]['summary'] = {
                'ran_cases': ran_cases,
                'passed_cases': passed_cases,
                'failed_cases': failed_cases,
                'total_run_time': total_run_time
            }
        return True

    def _replace_attribue_with_vaiablespool(self, obj):
        from Engine.config import VariablesPool
        for key in VariablesPool.__dict__.keys():
            if not re.match(r'_{2}', key):
                setattr(obj, key, getattr(VariablesPool, key))

    def _invoke_test_case(self, case_id):
        '''
        :returns: the test result, the log message, the running time
        :rtype: string, string, string
        '''
        start_time = time.time()
        try:
            case_classify = case_id.split('_')[0]
            # mod = __import__('{0}.{0}'.format(case_classify), fromlist=[case_id])
            mod = import_module('{0}.{0}'.format(case_classify))
            mod_class = getattr(mod, case_id)
            mod_class_inst = mod_class()
            self._replace_attribue_with_vaiablespool(mod_class_inst)
            run_result, log_message = getattr(mod_class_inst, 'run')()
            logger.debug('Run test case: {0}.{1}'.format(case_classify, case_id))
            end_time = time.time()
            run_time = end_time - start_time
            result = (run_result, log_message, int(run_time))

        except:
            logger.error('[RUN] {0}.{1}'.format(case_classify, case_id))
            logger.error('working directory: {0}'.format(os.getcwd()))
            logger.error(traceback.format_exc())
            end_time = time.time()
            run_time = end_time - start_time
            result = ('Error', str(traceback.format_exc()), int(run_time))

        return result[0], result[1], result[2]

    def run_all(self):
        self.run('.*')

    def run(self, specific_case_id):
        run_count = 0
        passed_num = 0
        failed_num = 0
        specific_case_id = '\w' if specific_case_id == '' else specific_case_id
        self.feedback.feedback_to_server(self.test_result)  # this can initial feedback server.

        if self.test_suites is False:
            return
        for case_classify in self.test_suites.keys():
            if case_classify is None:
                logger.error('Found a None case_classify!')
                continue

            total_run_time = 0
            for case_id in self.test_suites[case_classify]['ordered_cases']:
                # CaseID can support REGEX
                pattern = re.compile(specific_case_id)
                re_result = pattern.search(case_id)
                if re_result is None:  # skip test case
                    continue
                logger.debug('Pattern: {0}, Match case: {1}'.format(specific_case_id, case_id))

                # CSV can control which case could be run
                runable = self.test_suites[case_classify][case_id]['run']
                if runable == '1':     # filter run = 0
                    run_count += 1
                    try:
                        run_result, log_message, run_time = self._invoke_test_case(case_id)
                    except:
                        logger.error(traceback.format_exc())
                        continue

                    passed_num, failed_num = self._store_result(case_classify,
                                                                case_id,
                                                                run_result,
                                                                log_message,
                                                                passed_num, failed_num,
                                                                run_time)
                    total_run_time = total_run_time + run_time
                else:
                    self._store_result(case_classify, case_id, 'N/A', '', 0, 0, 0)

            else:
                csv_file_path = self.test_suites[case_classify]['csv_file_path']
                self._store_csv_path(case_classify, csv_file_path)  # save csv_file_path in test_result
                self._store_summary(case_classify, run_count, passed_num, failed_num, total_run_time)
                run_count = passed_num = failed_num = 0

        logger.debug('Test Result: %s' % str(self.test_result))

        try:
            self.feedback.feedback_to_server(self.test_result)
        except:
            logger.error(traceback.format_exc())
            logger.error('Please check the server configuration!')

        self.reporter.output_report(self.test_result)


def _write_template(script_filename_path, cases):
    if not os.path.exists(script_filename_path):    # Don't overwrite the test scripts
        fh = open(script_filename_path, 'w')
        if not fh:
            logger.error('Open file fail: {0}'.format(script_filename_path))

        for value in cases:
            temp = '''
class {0}:
    \'\'\'
    write doc here
    \'\'\'
    def __init__(self):
        pass
    def run(self):
        return False, \'\'
    '''.format(value)
            fh.write(temp)
        fh.close()


def GenerateTestCase(test_suites):
    working_folder = os.getcwd()
    test_cases_dir = {'top_level': working_folder}
    init_file = os.path.join(test_cases_dir['top_level'], '__init__.py')

    if not os.path.exists(init_file):
        fh = open(init_file, 'w')
        fh.close()
        logger.info('[GenerateTestCase] create file {0}'.format(init_file))
    else:
        logging.info('[GenerateTestCase] The ini file is exist: {0}'.format(init_file))

    for case_classify in test_suites:
        file_path = test_suites[case_classify]['csv_file_path']
        file_name = os.path.basename(file_path)
        file_name_no_ext = file_name.split('.')[0]
        test_cases_dir['layer2'] = os.path.join(test_cases_dir['top_level'], file_name_no_ext)
        script_filename_path = os.path.join(test_cases_dir['layer2'], file_name_no_ext + '.py')

        logger.info('[GenerateTestCase] script_filename_path = {0}'.format(script_filename_path))

        if not os.path.isfile(script_filename_path):
            try:
                os.makedirs(test_cases_dir['layer2'])             # create layer2 folder
            except:
                logger.error('canot make dir {0}'.format(script_filename_path))
                pass

            fh = open(os.path.join(test_cases_dir['layer2'], '__init__.py'), 'w')
            logger.info('[GenerateTestCase] create file {0}'.format(init_file))
            fh.close()

            _write_template(script_filename_path, test_suites[case_classify]['ordered_cases'])
        else:
            logging.warning('[GenerateTestCase] -The file is exist, it will not write template:')
            logging.warning('[GenerateTestCase] -{0}'.format(script_filename_path))



from parser import TestCaseParser
def ParseFromCSV(testCaseSuite):
    inst = TestCaseParser()
    return inst.ParseFromCSV(testCaseSuite)

import TestEngine
def GenerateTestCase(testCaseSuites, caseList, csvFileList):
    return TestEngine.GenerateTestCase(testCaseSuites, caseList, csvFileList)


def run(caseid_prefix):
    return TestEngine.run(caseid_prefix)


'''
Created on 2012/6/24

@author: kent
'''
import os
import logging
import csv
import sys
import traceback

logger = logging.getLogger('Parser')


def get_csv_files():
    '''Get all csv files in the working directory
    '''
    working_dir = os.getcwd()
    # testSuitesFolder = os.path.join(working_folder, 'TestSuites')
    # test_suites = os.listdir(testSuitesFolder)
    csv_files_path = []

    def find_csv_files(working_dir):
        item_list = os.listdir(working_dir)
        for item in item_list:
            if os.path.isdir(item):
                child_dir = os.path.join(working_dir, item)
                find_csv_files(child_dir)
            elif item[-3:] == 'csv':
                csv_files_path.append(os.path.join(working_dir, item))
            else:
                pass

    find_csv_files(working_dir)

    # ------
    # csvFilePaths = []
    # for entry1 in test_suites:
    #     entry1PathName = os.path.join(testSuitesFolder, entry1)
    #     if os.path.isdir(entry1PathName) == True:
    #         testSuites2Layer = os.listdir(entry1PathName)

    #         for entry2 in testSuites2Layer:
    #             entry2PathName = os.path.join(entry1PathName, entry2)
    #             if os.path.isdir(entry2PathName) == True:
    #                 logging.warning('[ParseFromCSV] There are folder in layer3 directory: {0}'.format(entry2PathName))
    #             else:
    #                 csvFilePaths.append(entry2PathName)
    #     else:
    #         csvFilePaths.append(entry1PathName)

    logger.info('[get_csv_files] CSV file list: {0}'.format(str(csv_files_path)))
    return csv_files_path


class TestCaseParser:
    def __init__(self):
        self.working_dir = os.getcwd()

    def _verify_csv_format(self, csv_file):
        '''
        [TODO] this function maybe not work.
        :param fh csv_file_fh: file handle of csv
        '''
        try:
            with open(csv_file, 'rb') as csv_file_fh:
                try:
                    CSVSniffer = csv.Sniffer()
                    DiaObject = CSVSniffer.sniff(csv_file_fh.readline())  # this will flush the buffer of csv content
                except:
                    logger.error('[VerifyCSVFormat] [%s] CSVSniffer fail, is this a csv?' % csv_file)
                    csv_file_fh.close()
                    sys.exit(0)

                # Check format of CSV file is validate
                if DiaObject.delimiter == ',':
                    result = True
                else:
                    logger.error('[VerifyCSVFormat] [%s] Delimiter must be ","' % csv_file)
                    logger.error('Delimiter: ' + str(DiaObject.delimiter))
                    result = False

                return result
        except IOError:
            logger.error(traceback.print_exc())
            logger.error('Fail to open file!')
            return False
        except:
            logger.error(traceback.format_exc())
            return False

    def _get_csv_content(self, csv_file_fh):
        '''
        :param fh csv_file_fh: file handle of csv
        '''
        try:
            csv_content = csv.reader(csv_file_fh, delimiter=',')
            logger.info(csv_content)
            result = csv_content
        except IOError as e:
            logger.error(e.strerror)
            result = False
        except csv.Error as e:
            logger.error('CSV file format error!')
            result = False

        return result

    def _extract_test_cases(self, case_classify, csv_content, csv_file):
        test_suites = {}
        ordered_cases = []
        test_suites[case_classify] = {}

        for row in csv_content:
            if csv_content.line_num == 1:  # skip title
                continue

            if len(row) > 0:
                case_id = row[1]
                case_run = row[4]
                test_suites[case_classify].update({case_id: {'run': case_run}})
                ordered_cases.append(case_id)  # store test case id in list

        test_suites[case_classify]['ordered_cases'] = ordered_cases
        test_suites[case_classify]['csv_file_path'] = csv_file

        return test_suites

    def parse_from_csv(self, test_suites=None):
        '''Parser CSV file and return the data with mapping structure, can parse 2 layers directory

        :return test_case_suite:
        - test_case_suite: dict type, whole test case information
            Dummy: {
                    csv_file_path: "<path>",
                    ordered_cases: [Dummy001, Dummy002],
                    Dummy001: { run: 1 },
                    Dummy002: { run: 0 }
        '''
        logger.info('Start parsing the CSV file to test cases {0}'.format(test_suites))

        csv_files = []
        if test_suites is None:
            csv_files = get_csv_files()
            logger.info('get csv files: %s' % str(csv_files))
        elif type(test_suites) == list:
            csv_files = test_suites
            logger.info('get csv files: %s' % str(csv_files))
        else:
            csv_files.append(test_suites)
            logger.info('get csv files: %s' % str(csv_files))

        full_test_cases = {}
        for csv_file in csv_files:
            if not self._verify_csv_format(csv_file):
                return False

            with open(csv_file, 'rb') as csv_file_fh:
                logger.info('[parse_from_csv] real_csv_files: {0}'.format(csv_file))

                csv_content = csv.reader(csv_file_fh, delimiter=',')
                if csv_content:
                    case_classify = os.path.basename(csv_file)[:-4]
                    full_test_cases.update(self._extract_test_cases(case_classify, csv_content, csv_file))
                else:
                    logger.warning('[parse_from_csv][{0}] is not a CSV file with comma'.format(csv_file))

        logger.debug('[parse_from_csv] all test cases: {0}'.format(full_test_cases))
        return full_test_cases


if __name__ == '__main__':
    pass




#!/usr/bin/env python
from setuptools import setup, find_packages

setup(
    name="pitester",
    version="1.0.0",
    # package_dir={'': 'src'},
    packages=['Engine'],
    # py_module=['pi_tester'],
    # scripts = [''],
    install_requires=['docutils>=0.3', 'junit-xml>=1.7'],
    package_data={},
    entry_points={
        'console_scripts': [
            'pitester=pitester:main'
        ]
    },
    py_modules=['pitester'],

    author="Dken",
    author_email="dken.tw@gmail.com",
    description="Automation test framework",
    license="MIT",
    keywords="automation test",
    url="https://github.com/dkentw/pi-tester",
)

if __name__ == '__main__':
    print find_packages()



FEEDBACK_SERVER = {
    "server_url": "http://localhost:8080",
    "reports_path": "/reports",
    "status_path": "/dashboard"
}

CLIENT = {
    "hostname": "KentChen"
}


class VariablesPool(object):
    def __setattr__(self, name, value):
        object.__setattr__(self, name, value)
